@@ -2,12 +2,11 @@
  * @file geis/geis.h
  * This is the public interface for the GEIS gesture API.
  *
- * Copyright 2010, 2011 Canonical Ltd.
+ * Copyright 2010-2013 Canonical Ltd.
  *
  * This library is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by the Free
- * Software Foundation; either version 3 of the License, or (at your option) any
- * later version.
+ * the terms of version 3 of the GNU Lesser General Public License as published
+ * by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -221,7 +220,117 @@ typedef enum GeisAttrType
 
 /* @} */
 
-/* Standard fundamental gesture attributes */
+/**
+ * @name Standard fundamental gesture attributes
+ *
+ * @{
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_ANGLE
+ * Angle covered by a gesture since it has started, in radians, counterclockwise.
+ * Its value ranges from -pi to pi.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_ANGLE_DELTA
+ * Angle covered by a gesture since its last update, in radians,
+ * counterclockwise.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_CENTROID_X
+ * This attribute provides the X coordinate of the centroid of the
+ * non-self-intersecting closed polygon defined by the touch points of the
+ * gesture, in device coordinates.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_CENTROID_Y
+ * This attribute provides the Y coordinate of the centroid of the
+ * non-self-intersecting closed polygon defined by the touch points of the
+ * gesture, in device coordinates.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_FOCUS_X
+ * This attribute provides the X coordinate of the focus point of a gesture,
+ * in screen coordinates.
+ * For direct devices (GEIS_DEVICE_ATTRIBUTE_DIRECT_TOUCH is GEIS_TRUE) it's
+ * the centroid point. For indirect devices it's the pointer/cursor position.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_FOCUS_Y
+ * This attribute provides the Y coordinate of the focus point of a gesture,
+ * in screen coordinates.
+ * For direct devices (GEIS_DEVICE_ATTRIBUTE_DIRECT_TOUCH is GEIS_TRUE) it's
+ * the centroid point. For indirect devices it's the pointer/cursor position.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_GESTURE_NAME
+ * Name of the gesture.
+ * This attribute is filled only when using GEIS v1 API (the simplified
+ * interface). On GEIS v2 this attribute has been replaced by the concept of
+ * gesture classes. Use geis_frame_is_class() instead.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_POSITION_X
+ * This attribute provides the X coordinate of the position of a gesture, in
+ * device coordinates. It's the same as the centroid of a gesture. See
+ * GEIS_GESTURE_ATTRIBUTE_CENTROID_X.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_POSITION_Y
+ * This attribute provides the Y coordinate of the position of a gesture, in
+ * device coordinates. It's the same as the centroid of a gesture. See
+ * GEIS_GESTURE_ATTRIBUTE_CENTROID_Y.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_TOUCH_0_ID
+ * This attribute provides the ID of the touch at index0.
+ * Only used by GEIS v1 API.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_TOUCH_0_X
+ * This attribute provides the X coordinate of the touch at index0.
+ * Only used by GEIS v1 API.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_TOUCH_0_Y
+ * This attribute provides the Y coordinate of the touch at index0.
+ * Only used by GEIS v1 API.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_TOUCH_1_ID
+ * This attribute provides the ID of the touch at index1.
+ * Only used by GEIS v1 API.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_TOUCH_1_X
+ * This attribute provides the X coordinate of the touch at index1.
+ * Only used by GEIS v1 API.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_TOUCH_1_Y
+ * This attribute provides the Y coordinate of the touch at index1.
+ * Only used by GEIS v1 API.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_TOUCH_2_ID
+ * This attribute provides the ID of the touch at index2.
+ * Only used by GEIS v1 API.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_TOUCH_2_X
+ * This attribute provides the X coordinate of the touch at index2.
+ * Only used by GEIS v1 API.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_TOUCH_2_Y
+ * This attribute provides the Y coordinate of the touch at index2.
+ * Only used by GEIS v1 API.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_TOUCH_3_ID
+ * This attribute provides the ID of the touch at index3.
+ * Only used by GEIS v1 API.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_TOUCH_3_X
+ * This attribute provides the X coordinate of the touch at index3.
+ * Only used by GEIS v1 API.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_TOUCH_3_Y
+ * This attribute provides the Y coordinate of the touch at index3.
+ * Only used by GEIS v1 API.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_TOUCH_4_ID
+ * This attribute provides the ID of the touch at index4.
+ * Only used by GEIS v1 API.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_TOUCH_4_X
+ * This attribute provides the X coordinate of the touch at index4.
+ * Only used by GEIS v1 API.
+ *
+ * @def GEIS_GESTURE_ATTRIBUTE_TOUCH_4_Y
+ * This attribute provides the Y coordinate of the touch at index4.
+ * Only used by GEIS v1 API.
+ */
 #define GEIS_GESTURE_ATTRIBUTE_ANGLE            "angle"
 #define GEIS_GESTURE_ATTRIBUTE_ANGLE_DELTA      "angle delta"
 #define GEIS_GESTURE_ATTRIBUTE_ANGULAR_VELOCITY "angular velocity"
@@ -265,7 +374,7 @@ typedef enum GeisAttrType
 #define GEIS_GESTURE_ATTRIBUTE_TOUCH_4_ID       "touch 4 id"
 #define GEIS_GESTURE_ATTRIBUTE_TOUCH_4_X        "touch 4 x"
 #define GEIS_GESTURE_ATTRIBUTE_TOUCH_4_Y        "touch 4 y"
-
+/* @} */
 
 /**
  * @defgroup geis_meta Initialization and Cleanup
@@ -422,7 +531,7 @@ GEIS_API GeisStatus geis_event_dispatch(GeisInstance geis_instance);
 
 /**
  * @class Geis
- * Represents an instance of the gestire recognition engine
+ * Represents an instance of the gesture recognition engine
  */
 /** @cond typedef */
 typedef struct _Geis *Geis;
@@ -447,11 +556,16 @@ typedef struct _Geis *Geis;
  * @def GEIS_INIT_TRACK_GESTURE_CLASSES
  * Tells GEIS to send gesture class events.
  * This initialization argument takes no parameters.
+ *
+ * @def GEIS_INIT_SYNCHRONOUS_START
+ * Performs all setup synchronously: geis_new() will block until all setup has
+ * completed successfully or unsuccessfully.
  */
 
 #define GEIS_INIT_SERVICE_PROVIDER     "org.libgeis.init.server"
 #define GEIS_INIT_TRACK_DEVICES        "org.libgeis.init.track-devices"
 #define GEIS_INIT_TRACK_GESTURE_CLASSES  "org.libgeis.init.track-gesture-classes"
+#define GEIS_INIT_SYNCHRONOUS_START      "org.libgeis.init.synchronous-start"
 
 /* @} */
 
@@ -459,19 +573,44 @@ typedef struct _Geis *Geis;
  * @name Vendor-defined Initialization Arguments
  *
  * @par
- * These initialization arguments are not a part of te GEIS specification and
+ * These initialization arguments are not a part of the GEIS specification and
  * may change.
  *
  * @{
  *
- * @def GEIS_INIT_OIF_MOCK_ENGINE
+ * @def GEIS_INIT_DBUS_BACKEND
+ * Uses the DBus back end (default).
  *
- * @def GEIS_INIT_OIF_XCB
+ * @def GEIS_INIT_GRAIL_BACKEND
+ * Uses the native grail back end (fallback).
+ *
+ * @def GEIS_INIT_XCB_BACKEND
+ * Uses the grail-embedded-in-X11 back end.
+ *
+ * @def GEIS_INIT_NO_ATOMIC_GESTURES
+ * Disables the use of (GEIS v1-style) atomic gestures:  only a single gesture
+ * is recognized at a time.
+ *
+ * @def GEIS_INIT_SEND_TENTATIVE_EVENTS
+ * Causes tentative events to be sent.  Tentative events indicate gestures may
+ * possibly be detected soon and allow early rejection if the events occur
+ * outside any area of interest.
+ *
+ * @def GEIS_INIT_SEND_SYNCHRONOS_EVENTS
+ * Causes all gesture events to be sent, even if there is zero apparent time
+ * difference between the events.  Normally all but the first consecutive event
+ * are discarded, since velocity values can not be calculated and multiple
+ * events between frame redraws are unlikely do have any value beyong consuming
+ * CPU.  Setting this init value will give the events to you if you really want
+ * them.
  */
 
-#define GEIS_INIT_OIF_MOCK_ENGINE "com.canonical.oif.mock.engine"
-#define GEIS_INIT_OIF_XCB         "com.canonical.oif.xcb"
-
+#define GEIS_INIT_DBUS_BACKEND           "com.canonical.oif.backend.dbus"
+#define GEIS_INIT_GRAIL_BACKEND          "com.canonical.oif.backend.grail"
+#define GEIS_INIT_XCB_BACKEND            "com.canonical.oif.backend.xcb"
+#define GEIS_INIT_NO_ATOMIC_GESTURES     "com.canonical.oif.no-atomic.gestures"
+#define GEIS_INIT_SEND_TENTATIVE_EVENTS  "com.canonical.oif.events.tentative"
+#define GEIS_INIT_SEND_SYNCHRONOS_EVENTS "com.canonical.oif.events.synchronous"
 /* @} */
 
 
@@ -490,7 +629,7 @@ typedef struct _Geis *Geis;
  * If no initialization arguments are passed, the parameter list consists of a
  * single NULL argument.
  */
-GEIS_API Geis geis_new(GeisString init_arg_name, ...);
+GEIS_API GEIS_VARARG Geis geis_new(GeisString init_arg_name, ...);
 
 /**
  * Cleans up an instance of the GEIS v2.0 API.
@@ -585,10 +724,91 @@ GEIS_API GeisString geis_error_message(Geis geis, GeisSize index);
  *
  * @{
  *
- * @def GEIS_CONFIG_OIF_MAX_EVENTS
+ * @def GEIS_CONFIG_MAX_EVENTS
  */
 
-#define GEIS_CONFIG_OIF_MAX_EVENTS  "com.canonical.oif.max_events"
+#define GEIS_CONFIG_MAX_EVENTS  "com.canonical.oif.max_events"
+
+/**
+ * @def GEIS_CONFIG_ATOMIC_GESTURES
+ * Indicates if atomic gestures are in use.  Value type GeisBoolean.
+ */
+#define GEIS_CONFIG_ATOMIC_GESTURES "com.canonical.use.atomic.gestures"
+
+/**
+ * @def GEIS_CONFIG_SEND_TENTATIVE_EVENTS
+ * See GEIS_INIT_SEND_TENTATIVE_EVENTS
+ */
+#define GEIS_CONFIG_SEND_TENTATIVE_EVENTS  "com.canonical.oif.events.tentative"
+
+/**
+ * @def GEIS_CONFIG_SEND_SYNCHRONOS_EVENTS
+ * See GEIS_INIT_SEND_SYNCHRONOS_EVENTS
+ */
+#define GEIS_CONFIG_SEND_SYNCHRONOS_EVENTS "com.canonical.oif.events.synchronous"
+
+/**
+ * @def GEIS_CONFIG_DRAG_THRESHOLD
+ * Movement threshold for recognizing a DRAG gesture (in meters).  Value type
+ * GeisFloat.
+ */
+#define GEIS_CONFIG_DRAG_THRESHOLD "com.canonical.oif.drag.threshold"
+
+/**
+ * @def GEIS_CONFIG_DRAG_TIMEOUT
+ * Timeout for recognizing a DRAG gesture (in milliseconds).  Value type
+ * GeisInteger.
+ */
+#define GEIS_CONFIG_DRAG_TIMEOUT "com.canonical.oif.drag.timeout"
+
+/**
+ * @def GEIS_CONFIG_PINCH_THRESHOLD
+ * Movement threshold for recognizing a PINCH gesture (in meters).  Value type
+ * GeisFloat.
+ */
+#define GEIS_CONFIG_PINCH_THRESHOLD "com.canonical.oif.pinch.threshold"
+
+/**
+ * @def GEIS_CONFIG_PINCH_TIMEOUT
+ * Timeout for recognizing a PINCH gesture (in milliseconds).  Value type
+ * GeisInteger.
+ */
+#define GEIS_CONFIG_PINCH_TIMEOUT "com.canonical.oif.pinch.timeout"
+
+/**
+ * @def GEIS_CONFIG_ROTATE_THRESHOLD
+ * Movement threshold for recognizing a ROTATE gesture (in meters).  Value type
+ * GeisFloat.
+ */
+#define GEIS_CONFIG_ROTATE_THRESHOLD "com.canonical.oif.rotate.threshold"
+
+/**
+ * @def GEIS_CONFIG_ROTATE_TIMEOUT
+ * Timeout for recognizing a ROTATE gesture (in milliseconds).  Value type
+ * GeisInteger.
+ */
+#define GEIS_CONFIG_ROTATE_TIMEOUT "com.canonical.oif.rotate.timeout"
+
+/**
+ * @def GEIS_CONFIG_TAP_THRESHOLD
+ * Movement threshold for recognizing a TAP gesture (in meters).  Value type
+ * GeisFloat.
+ */
+#define GEIS_CONFIG_TAP_THRESHOLD "com.canonical.oif.tap.threshold"
+
+/**
+ * @def GEIS_CONFIG_TAP_TIMEOUT
+ * Timeout for recognizing a TAP gesture (in milliseconds).  Value type
+ * GeisInteger.
+ */
+#define GEIS_CONFIG_TAP_TIMEOUT "com.canonical.oif.tap.timeout"
+
+/**
+ * @def GEIS_CONFIG_NUM_ACTIVE_SUBSCRIPTIONS
+ * The number of subscriptions currently active in the back end.
+ * This config is query-only and intended for unit test validations.
+ */
+#define GEIS_CONFIG_NUM_ACTIVE_SUBSCRIPTIONS "com.canonical.oif.debug.active_subs"
 
 /* @} */
 
@@ -709,7 +929,7 @@ typedef struct GeisGestureAttr
   /** The data type of the gesture attribute. */
   GeisAttrType type;
   /** The value of the attributes. */
-  union
+  __extension__ union
   {
     GeisBoolean boolean_val;
     GeisFloat   float_val;
@@ -885,6 +1105,9 @@ typedef enum _GeisEventType
   GEIS_EVENT_GESTURE_BEGIN            = 3000,
   GEIS_EVENT_GESTURE_UPDATE           = 3010,
   GEIS_EVENT_GESTURE_END              = 3020,
+  GEIS_EVENT_TENTATIVE_BEGIN          = 3500,
+  GEIS_EVENT_TENTATIVE_UPDATE         = 3510,
+  GEIS_EVENT_TENTATIVE_END            = 3520,
   GEIS_EVENT_INIT_COMPLETE            = 4000,
   GEIS_EVENT_USER_DEFINED             = 6000,
   GEIS_EVENT_ERROR                    = 7000
@@ -1155,6 +1378,21 @@ GEIS_API void geis_register_device_callback(Geis               geis,
                                             GeisEventCallback  event_callback,
                                             void              *context);
 
+/**
+ * Gets a cached device description for an identified device.
+ *
+ * @param[in]  geis      The GEIS API instance.
+ * @param[in]  device_id Identifies the device.
+ *
+ * The GEIS instance caches a list of gesture-capable input devices that have
+ * been reported.  The GeisDevice description for an identified device may be
+ * retrieved from that cache with this call.
+ *
+ * @returns a valid GeisDevice for the identified device, or NULL if no such
+ * device is in the cache.
+ */
+GEIS_API GeisDevice geis_get_device(Geis geis, GeisInteger device_id);
+
 /**
  * Adds a reference count to a device.
  * @memberof GeisDevice
@@ -1164,8 +1402,10 @@ GEIS_API void geis_register_device_callback(Geis               geis,
  * An application that wishes to guarantee the device object remains valid
  * should add a reference using this call, and unref when the object is no
  * longer needed.
+ *
+ * @returns @p device for syntactic convenience.
  */
-GEIS_API void geis_device_ref(GeisDevice device);
+GEIS_API GeisDevice geis_device_ref(GeisDevice device);
 
 /**
  * Removes a reference count from a device.
@@ -1214,6 +1454,15 @@ GEIS_API GeisSize geis_device_attr_count(GeisDevice device);
  */
 GEIS_API GeisAttr geis_device_attr(GeisDevice device, GeisSize index);
 
+/**
+ * Gets a named attribute from the device.
+ * @memberof GeisDevice
+ *
+ * @param[in] device    The device.
+ * @param[in] attr_name The name of the attribute to retrieve.
+ */
+GEIS_API GeisAttr geis_device_attr_by_name(GeisDevice device, GeisString attr_name);
+
 /* @} */
 
 /**
@@ -1302,7 +1551,7 @@ GEIS_API void geis_gesture_class_ref(GeisGestureClass gesture_class);
  *
  * @param[in] gesture_class  The gesture class object.
  *
- * The reference count of teh object is decremented and, if it reaches zero, the
+ * The reference count of the object is decremented and, if it reaches zero, the
  * object is destroyed.
  */
 GEIS_API void geis_gesture_class_unref(GeisGestureClass gesture_class);
@@ -1332,7 +1581,7 @@ GEIS_API GeisInteger geis_gesture_class_id(GeisGestureClass gesture_class);
 GEIS_API GeisSize geis_gesture_class_attr_count(GeisGestureClass gesture_class);
 
 /**
- * Gets the indicated attribute of teh gesture class.
+ * Gets the indicated attribute of the gesture class.
  * @memberof GeisGestureClass
  *
  * @param[in] gesture_class  The gesture class object.
@@ -1413,9 +1662,9 @@ typedef struct _GeisRegion *GeisRegion;
  *
  * @returns a newly created region, or NULL on failure.
  */
-GEIS_API GeisRegion geis_region_new(Geis       geis,
-                                    GeisString name,
-                                    GeisString init_arg_name, ...);
+GEIS_API GEIS_VARARG GeisRegion geis_region_new(Geis       geis,
+                                                GeisString name,
+                                                GeisString init_arg_name, ...);
 
 /**
  * Destroys a GEIS v2.0 region.
@@ -1462,7 +1711,7 @@ typedef struct _GeisFilter *GeisFilter;
 typedef enum _GeisFilterFacility
 {
   GEIS_FILTER_DEVICE   = 1000,  /**< Filters on device attributes. */
-  GEIS_FILTER_CLASS    = 2000,  /**< Filters on gesture attributes. */
+  GEIS_FILTER_CLASS    = 2000,  /**< Filters on gesture class and gesture attributes. */
   GEIS_FILTER_REGION   = 3000,  /**< Filters on region attributes. */
   GEIS_FILTER_SPECIAL  = 5000   /**< Filters on special attributes. */
 } GeisFilterFacility;
@@ -1533,10 +1782,27 @@ GEIS_API GeisString geis_filter_name(GeisFilter filter);
  * which the meaning of the filter-op and value depend on the type of the attr.
  *
  * The term description list must be terminated by a NULL.
+ *
+ * In the following example we add terms to filter drag gestures made with three
+ * touch points:
+ * @code
+ * geis_filter_add_term(filter,
+ *   GEIS_FILTER_CLASS,
+ *   GEIS_CLASS_ATTRIBUTE_NAME, GEIS_FILTER_OP_EQ, GEIS_GESTURE_DRAG,
+ *   GEIS_GESTURE_ATTRIBUTE_TOUCHES, GEIS_FILTER_OP_EQ, 3,
+ *   NULL);
+ * @endcode
+ *
+ * Term descriptions are usually ANDed together, so that specifying a class name
+ * and a number of touches will filter only for gestures that have both
+ * characteristics. But if you specify several class names (e.g. drag and pinch),
+ * those classes are ORed together instead. So you can receive events from a
+ * gesture that belongs to either drag, drag&pinch or only pinch classes.
+ * 
  */
-GEIS_API GeisStatus geis_filter_add_term(GeisFilter         filter,
-                                         GeisFilterFacility facility,
-                                         ...);
+GEIS_API GEIS_VARARG GeisStatus geis_filter_add_term(GeisFilter         filter,
+                                                     GeisFilterFacility facility,
+                                                     ...);
 
 /* @} */
 
@@ -1572,12 +1838,13 @@ typedef struct _GeisSubscription *GeisSubscription;
  * If this flag is not set, a new gesture will be identified for each change in
  * gesture class.
  */
-typedef enum GeisSubscriptionFlags
+enum
 {
   GEIS_SUBSCRIPTION_NONE = 0x0000,
   GEIS_SUBSCRIPTION_GRAB = 0x0001,
   GEIS_SUBSCRIPTION_CONT = 0x0002
-} GeisSubscriptionFlags;
+};
+typedef int GeisSubscriptionFlags;
 
 /**
  * Creates a new subscription.
@@ -1649,12 +1916,15 @@ GEIS_API GeisInteger geis_subscription_id(GeisSubscription subscription);
  * @param[in] subscription  The subscription.
  * @param[in] filter        The filter to be added to the subscription.
  *
- * Thye effect of filters are ANDed together so that, for example, a
- * subscription that has a filter for 3-finger gestures and drag gestures will
- * produce events only for 3-finger drag gestures.
+ * The effect of filters are ORed together so that, for example, a
+ * subscription that has a filter for 3-finger drag gestures and another for
+ * 2-finger pinch gestures will produce events for both 3-finger drag gestures
+ * and 2-finger pinch gestures.
  *
  * The default is no filters:  that is, all possible gesture events will be
  * reported.
+ *
+ * The subscription will take ownership of the filter.
  */
 GEIS_API GeisStatus geis_subscription_add_filter(GeisSubscription subscription,
                                                  GeisFilter       filter);
@@ -1678,10 +1948,51 @@ GEIS_API GeisFilter geis_subscription_filter_by_name(GeisSubscription sub,
  *
  * @param[in] subscription  The subscription.
  * @param[in] filter        The filter to be removed from the subscription.
+ *
+ * Ownership of the filter is passed to the caller.
  */
 GEIS_API GeisStatus geis_subscription_remove_filter(GeisSubscription subscription,
                                                     GeisFilter       filter);
 
+
+/**
+ * Gets a subscription-level configuration item.
+ *
+ * @param[in]  subscription      The subscription from which the configuration
+ *                               item will be retrieved.
+ * @param[in]  config_item_name  The name of the configuration item.
+ * @param[out] config_item_value A pointer to an appropriate variable to hold
+ *                               the retrieved config item value.
+ *
+ * Not all back ends support all configuration items.
+ *
+ * @retval GEIS_STATUS_BAD_ARGUMENT   an invalid argument value was passed
+ * @retval GEIS_STATUS_NOT_SUPPORTED  the configuration value is not supported
+ * @retval GEIS_STATUS_SUCCESS        normal successful completion
+ */
+GEIS_API GeisStatus geis_subscription_get_configuration(GeisSubscription subscription,
+                                                        GeisString       config_item_name,
+                                                        GeisPointer      config_item_value);
+
+/**
+ * Sets a subscription-level configuration item.
+ *
+ * @param[in] subscription      The subscription from which the configuration
+ *                              item will be retrieved.
+ * @param[in] config_item_name  The name of the configuration item.
+ * @param[in] config_item_value A pointer to an appropriate variable holding
+ *                              the config item value.
+ *
+ * Not all back ends support all configuration items.
+ *
+ * @retval GEIS_STATUS_BAD_ARGUMENT   an invalid argument value was passed
+ * @retval GEIS_STATUS_NOT_SUPPORTED  the configuration value is not supported
+ * @retval GEIS_STATUS_SUCCESS        normal successful completion
+ */
+GEIS_API GeisStatus geis_subscription_set_configuration(GeisSubscription subscription,
+                                                        GeisString       config_item_name,
+                                                        GeisPointer      config_item_value);
+
 /* @} */
 
 /**
@@ -1731,6 +2042,18 @@ typedef struct _GeisFrame    *GeisFrame;
  * GEIS_EVENT_GESTURE_END) should have two GEIS_ATTR_TYPE_POINTER attributes,
  * one containing a GeisGroupSet and one containing a GeisTouchSet.
  *
+ * For example: If four fingers are being simultaneously moved over a touchpad
+ * or touchscreen surface, Geis could start generating gesture events
+ * containing two groups: One group having a single frame from a four-fingers
+ * gesture of some class and a second group having two frames, each from a
+ * different two-fingers gesture (like one from a Rotate and the other from a
+ * Pinch gesture).
+ * This means that geis could interpret the movements of those four touch points as
+ * both a single four-fingers gesture and as two separate two-fingers gestures.
+ *
+ * There can be only a single frame per gesture in a gesture event. I.e. no two
+ * frames will return the same GeisGestureId in the same gesture event.
+ *
  * @{
  *
  * @def GEIS_EVENT_ATTRIBUTE_GROUPSET
@@ -1738,9 +2061,27 @@ typedef struct _GeisFrame    *GeisFrame;
  *
  * @def GEIS_EVENT_ATTRIBUTE_TOUCHSET
  * The event attribute containing a pointer to a GeisTouchSet.
+ *
+ * @def GEIS_EVENT_ATTRIBUTE_CONSTRUCTION_FINISHED
+ * Event attribute containing a boolean.
+ * This property allows the client to determine if all the possible gestures
+ * from the set of touches in this event have already been presented. When
+ * this value is true, the client will have received all the information needed
+ * to make a gesture accept and reject decision based on potentially
+ * overlapping gestures. An example is when both one and two touch gestures are
+ * subscribed on the same window with the same gesture classes and thresholds.
+ * When this property is true for one touch gesture events, the client can be
+ * sure there are no other touches unless a two touch gesture event has already
+ * been sent.
+ * Another example is when you subscribe for three touches Touch and four
+ * touches Drag. As soon as a third finger is detected a three touches Touch
+ * gesture will begin, but you cannot be sure a fourth finger isn't coming
+ * right after (that can eventually cause a four touches Drag) until this
+ * property is true.
  */
 #define GEIS_EVENT_ATTRIBUTE_GROUPSET  "group set"
 #define GEIS_EVENT_ATTRIBUTE_TOUCHSET  "touch set"
+#define GEIS_EVENT_ATTRIBUTE_CONSTRUCTION_FINISHED  "construction finished"
 
 /* @} */
 
@@ -1752,6 +2093,11 @@ typedef struct _GeisFrame    *GeisFrame;
  * is capable of reporting differing sets of touch attributes, so there is no
  * guarantee that any or all of the defined touch attributes will bre present.
  *
+ * If the touch comes from a direct device (see
+ * GEIS_DEVICE_ATTRIBUTE_DIRECT_TOUCH) its position (x and y attributes) will
+ * be in window coordinates, otherwise it will be in the input device's own
+ * coordinate system.
+ *
  * @{
  *
  * @def GEIS_TOUCH_ATTRIBUTE_ID
@@ -1889,6 +2235,8 @@ GEIS_API GeisAttr geis_touch_attr_by_name(GeisTouch touch, GeisString name);
  * @memberof GeisFrame
  *
  * @param[in] frame  the gesture frame.
+ *
+ * @returns the identifier of the gesture to which the given frame belongs.
  */
 GEIS_API GeisGestureId geis_frame_id(GeisFrame frame);
 
@@ -1927,9 +2275,14 @@ GEIS_API GeisAttr geis_frame_attr(GeisFrame frame, GeisSize index);
  * @memberof GeisFrame
  *
  * @param[in] frame  The gesture frame.
- * @param[in] name   Names the attr to retrieve.
- *
+ * @param[in] name   Names the attr to retrieve, such as one of the
+ *                   GEIS_GESTURE_ATTRIBUTE_* constants.
  * @returns the named attr if it is present, NULL otherwise.
+ *
+ * Usage example:
+ * @code
+ * GeisAttr angle = geis_frame_attr_by_name(frame, GEIS_GESTURE_ATTRIBUTE_ANGLE);
+ * @endcode
  */
 GEIS_API GeisAttr geis_frame_attr_by_name(GeisFrame frame, GeisString name);
 
@@ -1959,12 +2312,32 @@ GEIS_API GeisSize geis_frame_touchid_count(GeisFrame frame);
 GEIS_API GeisTouchId geis_frame_touchid(GeisFrame frame, GeisSize index);
 
 /**
- * Marks a gesture as rejected.
- * @memberof GeisFrame
+ * Marks a gesture as accepted.
  *
+ * @param[in] geis        The GEIS instance.
+ * @param[in] group       The gesture group containing the accepted gesture.
  * @param[in] gesture_id  Identifies the gesture.
+ *
+ * @sa geis_frame_id
  */
-GEIS_API void geis_gesture_reject(GeisGestureId gesture_id);
+GEIS_API GeisStatus geis_gesture_accept(Geis          geis,
+                                        GeisGroup     group,
+                                        GeisGestureId gesture_id);
+
+/**
+ * Marks a gesture as rejected.
+ *
+ * @param[in] geis        The GEIS instance.
+ * @param[in] group       The gesture group containing the rejected gesture.
+ * @param[in] gesture_id  Identifies the gesture.
+ *
+ * After you reject a gesture you no longer get its frames.
+ *
+ * @sa geis_frame_id
+ */
+GEIS_API GeisStatus geis_gesture_reject(Geis          geis,
+                                        GeisGroup     group,
+                                        GeisGestureId gesture_id);
 
 /* @} */
 
